@@ -10,6 +10,7 @@ import types
 import copy
 
 from contextlib import contextmanager
+from functools import wraps
 
 import numpy as np
 import param
@@ -17,7 +18,6 @@ import pandas as pd # noqa
 
 from param.parameterized import add_metaclass, ParameterizedMetaclass
 
-from .. import util
 from ..accessors import Redim
 from ..dimension import (
     Dimension, Dimensioned, LabelledData, dimension_name, process_dimensions
@@ -36,13 +36,18 @@ from .interface import Interface, iloc, ndloc # noqa (API import)
 from .multipath import MultiInterface         # noqa (API import)
 from .image import ImageInterface             # noqa (API import)
 from .pandas import PandasInterface           # noqa (API import)
-from .spatialpandas import SpatialPandasInterface # noqa (API import)
+from .spatialpandas import SpatialPandasInterface     # noqa (API import)
+from .spatialpandas_dask import DaskSpatialPandasInterface # noqa (API import)
 from .xarray import XArrayInterface           # noqa (API import)
 
+# Ensures correct holoviews.core.util is sourced
+from .. import util
+
 default_datatype = 'dataframe'
 
-datatypes = ['dataframe', 'dictionary', 'grid', 'xarray', 'dask',
-             'cuDF', 'spatialpandas', 'array', 'multitabular', 'ibis']
+datatypes = ['dataframe', 'dictionary', 'grid', 'xarray', 'multitabular',
+             'spatialpandas', 'dask_spatialpandas', 'dask', 'cuDF', 'array',
+             'ibis']
 
 
 def concat(datasets, datatype=None):
@@ -189,6 +194,7 @@ class PipelineMeta(ParameterizedMetaclass):
 
     @staticmethod
     def pipelined(method_fn, method_name):
+        @wraps(method_fn)
         def pipelined_fn(*args, **kwargs):
             from ...operation.element import method as method_op
             inst = args[0]
@@ -235,8 +241,6 @@ class PipelineMeta(ParameterizedMetaclass):
                     inst._in_method = False
             return result
 
-        pipelined_fn.__doc__ = method_fn.__doc__
-
         return pipelined_fn
 
 
@@ -903,6 +907,11 @@ argument to specify a selection specification""")
                 transformed = transformed.collapse()
             return transformed.clone(new_type=type(self))
 
+        ndims = len(dimensions)
+        min_d, max_d = self.param.objects('existing')['kdims'].bounds
+        generic_type = (min_d is not None and ndims < min_d) or (max_d is not None and ndims > max_d)
+        new_type = Dataset if generic_type else None
+
         # Handle functions
         kdims = [self.get_dimension(d, strict=True) for d in dimensions]
         if not self:
@@ -911,17 +920,15 @@ argument to specify a selection specification""")
                 vdims = [d for vd in self.vdims for d in [vd, vd.clone('_'.join([vd.name, spread_name]))]]
             else:
                 vdims = self.vdims
-            return self.clone([], kdims=kdims, vdims=vdims)
+            if not kdims and len(vdims) == 1:
+                return np.nan
+            return self.clone([], kdims=kdims, vdims=vdims, new_type=new_type)
 
         vdims = self.vdims
         aggregated, dropped = self.interface.aggregate(self, kdims, function, **kwargs)
         aggregated = self.interface.unpack_scalar(self, aggregated)
         vdims = [vd for vd in vdims if vd not in dropped]
 
-        ndims = len(dimensions)
-        min_d, max_d = self.param.objects('existing')['kdims'].bounds
-        generic_type = (min_d is not None and ndims < min_d) or (max_d is not None and ndims > max_d)
-
         if spreadfn:
             error, _ = self.interface.aggregate(self, dimensions, spreadfn)
             spread_name = spreadfn.__name__
@@ -934,7 +941,7 @@ argument to specify a selection specification""")
                 idx = vdims.index(d)
                 combined = combined.add_dimension(dim, idx+1, dvals, True)
                 vdims = combined.vdims
-            return combined.clone(new_type=Dataset if generic_type else type(self))
+            return combined.clone(new_type=new_type)
 
         if np.isscalar(aggregated):
             return aggregated
@@ -945,8 +952,7 @@ argument to specify a selection specification""")
             except:
                 datatype = self.param.objects('existing')['datatype'].default
                 return self.clone(aggregated, kdims=kdims, vdims=vdims,
-                                  new_type=Dataset if generic_type else None,
-                                  datatype=datatype)
+                                  new_type=new_type, datatype=datatype)
 
 
     def groupby(self, dimensions=[], container_type=HoloMap, group_type=None,
@@ -1208,17 +1214,17 @@ argument to specify a selection specification""")
 
     # Overrides of superclass methods that are needed so that PipelineMeta
     # will find them to wrap with pipeline support
+    @wraps(Dimensioned.options)
     def options(self, *args, **kwargs):
         return super(Dataset, self).options(*args, **kwargs)
-    options.__doc__ = Dimensioned.options.__doc__
 
+    @wraps(LabelledData.map)
     def map(self, *args, **kwargs):
         return super(Dataset, self).map(*args, **kwargs)
-    map.__doc__ = LabelledData.map.__doc__
 
+    @wraps(LabelledData.relabel)
     def relabel(self, *args, **kwargs):
         return super(Dataset, self).relabel(*args, **kwargs)
-    relabel.__doc__ = LabelledData.relabel.__doc__
 
     @property
     def iloc(self):
