@@ -8,7 +8,7 @@ import param
 
 from holoviews.core.spaces import DynamicMap
 from holoviews.core.util import LooseVersion, pd
-from holoviews.element import Points, Scatter, Curve, Histogram
+from holoviews.element import Points, Scatter, Curve, Histogram, Polygons
 from holoviews.element.comparison import ComparisonTestCase
 from holoviews.streams import * # noqa (Test all available streams)
 from holoviews.util import Dynamic, extension
@@ -28,6 +28,17 @@ def test_all_stream_parameters_constant():
                                 % (name, stream_cls.__name__))
 
 
+def test_all_linked_stream_parameters_owners():
+    "Test to ensure operations can accept parameters in streams dictionary"
+    stream_classes = param.concrete_descendents(LinkedStream)
+    for stream_class in stream_classes.values():
+        for name, p in stream_class.param.params().items():
+            if name != 'name' and (p.owner != stream_class):
+                msg = ("Linked stream %r has parameter %r which is "
+                       "inherited from %s. Parameter needs to be redeclared "
+                       "in the class definition of this linked stream.")
+                raise Exception(msg % (stream_class, name, p.owner))
+
 class TestStreamsDefine(ComparisonTestCase):
 
     def setUp(self):
@@ -107,7 +118,7 @@ class TestStreamsDefine(ComparisonTestCase):
         self.assertEqual(self.ExplicitTest.param['test'].doc, 'Test docstring')
 
 
-class TestSubscriber(object):
+class _TestSubscriber(object):
 
     def __init__(self, cb=None):
         self.call_count = 0
@@ -358,6 +369,46 @@ class TestParamMethodStream(ComparisonTestCase):
         inner.x = 10
         self.assertEqual(dmap[()], Points([10]))
 
+
+    def test_param_instance_steams_dict(self):
+        inner = self.inner()
+
+        def test(x):
+            return Points([x])
+
+        dmap = DynamicMap(test, streams=dict(x=inner.param.x))
+
+        inner.x = 10
+        self.assertEqual(dmap[()], Points([10]))
+
+    def test_param_class_steams_dict(self):
+        class ClassParamExample(param.Parameterized):
+            x = param.Number(default=1)
+
+        def test(x):
+            return Points([x])
+
+        dmap = DynamicMap(test, streams=dict(x=ClassParamExample.param.x))
+
+        ClassParamExample.x = 10
+        self.assertEqual(dmap[()], Points([10]))
+
+    def test_panel_param_steams_dict(self):
+        try:
+            import panel
+        except:
+            raise SkipTest('Panel required for widget support in streams dict')
+        widget = panel.widgets.FloatSlider(value=1)
+
+        def test(x):
+            return Points([x])
+
+        dmap = DynamicMap(test, streams=dict(x=widget))
+
+        widget.value = 10
+        self.assertEqual(dmap[()], Points([10]))
+
+
     def test_param_method_depends_no_deps(self):
         inner = self.inner()
         stream = ParamMethod(inner.method_no_deps)
@@ -510,22 +561,22 @@ class TestParamMethodStream(ComparisonTestCase):
 class TestSubscribers(ComparisonTestCase):
 
     def test_exception_subscriber(self):
-        subscriber = TestSubscriber()
+        subscriber = _TestSubscriber()
         position = PointerXY(subscribers=[subscriber])
         kwargs = dict(x=3, y=4)
         position.event(**kwargs)
         self.assertEqual(subscriber.kwargs, kwargs)
 
     def test_subscriber_disabled(self):
-        subscriber = TestSubscriber()
+        subscriber = _TestSubscriber()
         position = PointerXY(subscribers=[subscriber])
         kwargs = dict(x=3, y=4)
         position.update(**kwargs)
         self.assertEqual(subscriber.kwargs, None)
 
     def test_subscribers(self):
-        subscriber1 = TestSubscriber()
-        subscriber2 = TestSubscriber()
+        subscriber1 = _TestSubscriber()
+        subscriber2 = _TestSubscriber()
         position = PointerXY(subscribers=[subscriber1, subscriber2])
         kwargs = dict(x=3, y=4)
         position.event(**kwargs)
@@ -533,7 +584,7 @@ class TestSubscribers(ComparisonTestCase):
         self.assertEqual(subscriber2.kwargs, kwargs)
 
     def test_batch_subscriber(self):
-        subscriber = TestSubscriber()
+        subscriber = _TestSubscriber()
 
         positionX = PointerX(subscribers=[subscriber])
         positionY = PointerY(subscribers=[subscriber])
@@ -546,8 +597,8 @@ class TestSubscribers(ComparisonTestCase):
         self.assertEqual(subscriber.call_count, 1)
 
     def test_batch_subscribers(self):
-        subscriber1 = TestSubscriber()
-        subscriber2 = TestSubscriber()
+        subscriber1 = _TestSubscriber()
+        subscriber2 = _TestSubscriber()
 
         positionX = PointerX(subscribers=[subscriber1, subscriber2])
         positionY = PointerY(subscribers=[subscriber1, subscriber2])
@@ -572,7 +623,7 @@ class TestSubscribers(ComparisonTestCase):
         dmap = DynamicMap(points, streams=[stream])
         def cb():
             dmap[()]
-        subscriber = TestSubscriber(cb)
+        subscriber = _TestSubscriber(cb)
         stream.add_subscriber(subscriber)
         dmap[()]
         stream.send(1)
@@ -805,8 +856,8 @@ class TestBufferDictionaryStream(ComparisonTestCase):
     def test_buffer_dict_send_verify_column_fail(self):
         data = {'x': np.array([0]), 'y': np.array([1])}
         buff = Buffer(data)
-        error = "Input expected to have columns \['x', 'y'\], got \['x'\]"
-        with self.assertRaisesRegexp(IndexError, error):
+        error = r"Input expected to have columns \['x', 'y'\], got \['x'\]"
+        with self.assertRaisesRegex(IndexError, error):
             buff.send({'x': np.array([2])})
 
     def test_buffer_dict_send_verify_shape_fail(self):
@@ -866,8 +917,8 @@ class TestBufferDataFrameStream(ComparisonTestCase):
     def test_buffer_dframe_send_verify_column_fail(self):
         data = pd.DataFrame({'x': np.array([0]), 'y': np.array([1])})
         buff = Buffer(data, index=False)
-        error = "Input expected to have columns \['x', 'y'\], got \['x'\]"
-        with self.assertRaisesRegexp(IndexError, error):
+        error = r"Input expected to have columns \['x', 'y'\], got \['x'\]"
+        with self.assertRaisesRegex(IndexError, error):
             buff.send(pd.DataFrame({'x': np.array([2])}))
 
     def test_clear_buffer_dframe_with_index(self):
@@ -877,6 +928,171 @@ class TestBufferDataFrameStream(ComparisonTestCase):
         self.assertEqual(buff.data, data.iloc[:0, :].reset_index())
 
 
+class Sum(Derived):
+    v = param.Number(constant=True)
+
+    def __init__(self, val_streams, exclusive=False, base=0):
+        self.base = base
+        super(Sum, self).__init__(input_streams=val_streams, exclusive=exclusive)
+
+    @property
+    def constants(self):
+        return dict(base=self.base)
+
+    @classmethod
+    def transform_function(cls, stream_values, constants):
+        v = sum([val["v"] for val in stream_values if val["v"]])
+        return dict(v=v + constants['base'])
+
+
+Val = Stream.define("Val", v=0.0)
+
+
+class TestDerivedStream(ComparisonTestCase):
+
+    def test_simple_derived_stream(self):
+        # Define input streams
+        v0 = Val(v=1.0)
+        v1 = Val(v=2.0)
+
+        # Build Sum stream
+        s0 = Sum([v0, v1])
+
+        # Check outputs
+        self.assertEqual(s0.v, 3.0)
+
+        # Update v0
+        v0.event(v=7.0)
+        self.assertEqual(s0.v, 9.0)
+
+        # Update v1
+        v1.event(v=-8.0)
+        self.assertEqual(s0.v, -1.0)
+
+    def test_nested_derived_stream(self):
+        v0 = Val(v=1.0)
+        v1 = Val(v=4.0)
+        v2 = Val(v=7.0)
+
+        # Build nested sum stream
+        s1 = Sum([v0, v1])
+        s0 = Sum([s1, v2])
+
+        # Check initial value
+        self.assertEqual(s0.v, 12.0)
+
+        # Update top-level value
+        v2.event(v=8.0)
+        self.assertEqual(s0.v, 13.0)
+
+        # Update nested value
+        v1.event(v=5.0,)
+        self.assertEqual(s0.v, 14.0)
+
+    def test_derived_stream_constants(self):
+        v0 = Val(v=1.0)
+        v1 = Val(v=4.0)
+        v2 = Val(v=7.0)
+
+        # Build Sum stream with base value
+        s0 = Sum([v0, v1, v2], base=100)
+
+        # Check initial value
+        self.assertEqual(s0.v, 112.0)
+
+        # Update value
+        v2.event(v=8.0)
+        self.assertEqual(s0.v, 113.0)
+
+    def test_exclusive_derived_stream(self):
+        # Define input streams
+        v0 = Val()
+        v1 = Val(v=2.0)
+
+        # Build exclusive Sum stream
+        # In this case, all streams except the most recently updated will be reset on
+        # update
+        s0 = Sum([v0, v1], exclusive=True)
+
+        # Check outputs
+        self.assertEqual(s0.v, 2.0)
+
+        # Update v0
+        v0.event(v=7.0)
+        self.assertEqual(s0.v, 7.0)
+
+        # Update v1
+        v1.event(v=-8.0)
+        self.assertEqual(s0.v, -8.0)
+
+
+class TestHistoryStream(ComparisonTestCase):
+    def test_initial_history_stream_values(self):
+        # Check values list is initialized with initial contents of input stream
+        val = Val(v=1.0)
+        history = History(val)
+        self.assertEqual(history.contents, {"values": [val.contents]})
+
+    def test_history_stream_values_appended(self):
+        val = Val(v=1.0)
+        history = History(val)
+        # Perform a few updates on val stream
+        val.event(v=2.0)
+        val.event(v=3.0)
+        self.assertEqual(
+            history.contents,
+            {"values": [{"v": 1.0}, {"v": 2.0}, {"v": 3.0}]}
+        )
+
+        # clear
+        history.clear_history()
+        self.assertEqual(history.contents, {"values": []})
+
+    def test_history_stream_trigger_callbacks(self):
+        # Construct history stream
+        val = Val(v=1.0)
+        history = History(val)
+
+        # Register callback
+        callback_input = []
+        def cb(**kwargs):
+            callback_input.append(kwargs)
+        history.add_subscriber(cb)
+        self.assertEqual(callback_input, [])
+
+        # Perform updates on val stream and make sure history callback is triggered
+        del callback_input[:]
+        val.event(v=2.0)
+        self.assertEqual(
+            callback_input[0],
+            {"values": [{"v": 1.0}, {"v": 2.0}]}
+        )
+
+        del callback_input[:]
+        val.event(v=3.0)
+        self.assertEqual(
+            callback_input[0],
+            {"values": [{"v": 1.0}, {"v": 2.0}, {"v": 3.0}]}
+        )
+
+        # clearing history should trigger callback
+        del callback_input[:]
+        history.clear_history()
+        history.event()
+        self.assertEqual(
+            callback_input[0],
+            {"values": []}
+        )
+
+        # Update after clearing
+        del callback_input[:]
+        val.event(v=4.0)
+        self.assertEqual(
+            callback_input[0],
+            {"values": [{"v": 4.0}]}
+        )
+
+
 class TestExprSelectionStream(ComparisonTestCase):
 
     def setUp(self):
@@ -889,13 +1105,15 @@ class TestExprSelectionStream(ComparisonTestCase):
             expr_stream = SelectionExpr(element)
 
             # Check stream properties
-            self.assertEqual(len(expr_stream._source_streams), 1)
-            self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+            self.assertEqual(len(expr_stream.input_streams), 3)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsInstance(expr_stream.input_streams[1], Lasso)
+            self.assertIsInstance(expr_stream.input_streams[2], Selection1D)
             self.assertIsNone(expr_stream.bbox)
             self.assertIsNone(expr_stream.selection_expr)
 
             # Simulate interactive update by triggering source stream
-            expr_stream._source_streams[0].event(bounds=(1, 1, 3, 4))
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
 
             # Check SelectionExpr values
             self.assertEqual(
@@ -914,13 +1132,15 @@ class TestExprSelectionStream(ComparisonTestCase):
             expr_stream = SelectionExpr(element)
 
             # Check stream properties
-            self.assertEqual(len(expr_stream._source_streams), 1)
-            self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+            self.assertEqual(len(expr_stream.input_streams), 3)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsInstance(expr_stream.input_streams[1], Lasso)
+            self.assertIsInstance(expr_stream.input_streams[2], Selection1D)
             self.assertIsNone(expr_stream.bbox)
             self.assertIsNone(expr_stream.selection_expr)
 
             # Simulate interactive update by triggering source stream
-            expr_stream._source_streams[0].event(bounds=(1, 1, 3, 4))
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
 
             # Check SelectionExpr values
             self.assertEqual(
@@ -943,13 +1163,15 @@ class TestExprSelectionStream(ComparisonTestCase):
             expr_stream = SelectionExpr(element)
 
             # Check stream properties
-            self.assertEqual(len(expr_stream._source_streams), 1)
-            self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+            self.assertEqual(len(expr_stream.input_streams), 3)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+            self.assertIsInstance(expr_stream.input_streams[1], Lasso)
+            self.assertIsInstance(expr_stream.input_streams[2], Selection1D)
             self.assertIsNone(expr_stream.bbox)
             self.assertIsNone(expr_stream.selection_expr)
 
             # Simulate interactive update by triggering source stream
-            expr_stream._source_streams[0].event(bounds=(3, 4, 1, 1))
+            expr_stream.input_streams[0].event(bounds=(3, 4, 1, 1))
 
             # Check SelectionExpr values
             self.assertEqual(
@@ -967,30 +1189,29 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream = SelectionExpr(hist)
 
         # Check stream properties
-        self.assertEqual(len(expr_stream._source_streams), 1)
-        self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+        self.assertEqual(len(expr_stream.input_streams), 1)
+        self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
         self.assertIsNone(expr_stream.bbox)
         self.assertIsNone(expr_stream.selection_expr)
 
         # Simulate interactive update by triggering source stream.
         # Select second and forth bar.
-        expr_stream._source_streams[0].event(bounds=(1.5, 2.5, 4.6, 6))
+        expr_stream.input_streams[0].event(bounds=(1.5, 2.5, 4.6, 6))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('x').digitize(hist.edges).isin([2, 4]))
+            repr((dim('x')>=1.5)&(dim('x')<=4.6))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.6)})
 
         # Select third, forth, and fifth bar.  Make sure there is special
         # handling when last bar is selected to include values exactly on the
         # upper edge in the selection
-        expr_stream._source_streams[0].event(bounds=(2.5, -10, 8, 10))
+        expr_stream.input_streams[0].event(bounds=(2.5, -10, 8, 10))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr((dim('x').digitize(hist.edges).isin([3, 4, 5])) |
-                 (dim('x') == hist.edges[-1]))
+            repr((dim('x')>=2.5)&(dim('x')<=8))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (2.5, 5.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (2.5, 8)})
 
     def test_selection_expr_stream_hist_invert_axes(self):
         # Create SelectionExpr on element
@@ -1000,30 +1221,29 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream = SelectionExpr(hist)
 
         # Check stream properties
-        self.assertEqual(len(expr_stream._source_streams), 1)
-        self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+        self.assertEqual(len(expr_stream.input_streams), 1)
+        self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
         self.assertIsNone(expr_stream.bbox)
         self.assertIsNone(expr_stream.selection_expr)
 
         # Simulate interactive update by triggering source stream.
         # Select second and forth bar.
-        expr_stream._source_streams[0].event(bounds=(2.5, 1.5, 6, 4.6))
+        expr_stream.input_streams[0].event(bounds=(2.5, 1.5, 6, 4.6))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('x').digitize(hist.edges).isin([2, 4]))
+            repr((dim('x')>=1.5)&(dim('x')<=4.6))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.6)})
 
         # Select third, forth, and fifth bar.  Make sure there is special
         # handling when last bar is selected to include values exactly on the
         # upper edge in the selection
-        expr_stream._source_streams[0].event(bounds=(-10, 2.5, 10, 8))
+        expr_stream.input_streams[0].event(bounds=(-10, 2.5, 10, 8))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr((dim('x').digitize(hist.edges).isin([3, 4, 5])) |
-                 (dim('x') == hist.edges[-1]))
+            repr((dim('x')>=2.5)&(dim('x')<=8))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (2.5, 5.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (2.5, 8)})
 
     def test_selection_expr_stream_hist_invert_xaxis_yaxis(self):
         # Create SelectionExpr on element
@@ -1034,30 +1254,87 @@ class TestExprSelectionStream(ComparisonTestCase):
         expr_stream = SelectionExpr(hist)
 
         # Check stream properties
-        self.assertEqual(len(expr_stream._source_streams), 1)
-        self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+        self.assertEqual(len(expr_stream.input_streams), 1)
+        self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
         self.assertIsNone(expr_stream.bbox)
         self.assertIsNone(expr_stream.selection_expr)
 
         # Simulate interactive update by triggering source stream.
         # Select second and forth bar.
-        expr_stream._source_streams[0].event(bounds=(4.6, 6, 1.5, 2.5))
+        expr_stream.input_streams[0].event(bounds=(4.6, 6, 1.5, 2.5))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr(dim('x').digitize(hist.edges).isin([2, 4]))
+            repr((dim('x')>=1.5)&(dim('x')<=4.6))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (1.5, 4.6)})
 
         # Select third, forth, and fifth bar.  Make sure there is special
         # handling when last bar is selected to include values exactly on the
         # upper edge in the selection
-        expr_stream._source_streams[0].event(bounds=(8, 10, 2.5, -10))
+        expr_stream.input_streams[0].event(bounds=(8, 10, 2.5, -10))
         self.assertEqual(
             repr(expr_stream.selection_expr),
-            repr((dim('x').digitize(hist.edges).isin([3, 4, 5])) |
-                 (dim('x') == hist.edges[-1]))
+            repr((dim('x')>=2.5)&(dim('x')<=8))
         )
-        self.assertEqual(expr_stream.bbox, {'x': (2.5, 5.5)})
+        self.assertEqual(expr_stream.bbox, {'x': (2.5, 8)})
+
+
+    def test_selection_expr_stream_polygon_index_cols(self):
+        # Create SelectionExpr on element
+        try: import shapely # noqa
+        except:
+            try: import spatialpandas # noqa
+            except: raise SkipTest('Shapely required for polygon selection')
+        poly = Polygons([
+            [(0, 0, 'a'), (2, 0, 'a'), (1, 1, 'a')],
+            [(2, 0, 'b'), (4, 0, 'b'), (3, 1, 'b')],
+            [(1, 1, 'c'), (3, 1, 'c'), (2, 2, 'c')]
+        ], vdims=['cat'])
+
+        events = []
+        expr_stream = SelectionExpr(poly, index_cols=['cat'])
+        expr_stream.add_subscriber(lambda **kwargs: events.append(kwargs))
+
+        # Check stream properties
+        self.assertEqual(len(expr_stream.input_streams), 3)
+        self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
+        self.assertIsInstance(expr_stream.input_streams[1], Lasso)
+        self.assertIsInstance(expr_stream.input_streams[2], Selection1D)
+        self.assertIsNone(expr_stream.bbox)
+        self.assertIsNone(expr_stream.selection_expr)
+
+        expr_stream.input_streams[2].event(index=[0, 1])
+        self.assertEqual(
+            repr(expr_stream.selection_expr),
+            repr(dim('cat').isin(['a', 'b']))
+        )
+        self.assertEqual(expr_stream.bbox, None)
+        self.assertEqual(len(events), 1)
+
+        # Ensure bounds event does not trigger another update
+        expr_stream.input_streams[0].event(bounds=(0, 0, 4, 1))
+        self.assertEqual(
+            repr(expr_stream.selection_expr),
+            repr(dim('cat').isin(['a', 'b']))
+        )
+        self.assertEqual(len(events), 1)
+
+        # Ensure geometry event does trigger another update
+        expr_stream.input_streams[1].event(geometry=np.array([(0, 0), (4, 0), (4, 2), (0, 2)]))
+        self.assertEqual(
+            repr(expr_stream.selection_expr),
+            repr(dim('cat').isin(['a', 'b', 'c']))
+        )
+        self.assertEqual(len(events), 2)
+
+        # Ensure index event does trigger another update
+        expr_stream.input_streams[2].event(index=[1, 2])
+        self.assertEqual(
+            repr(expr_stream.selection_expr),
+            repr(dim('cat').isin(['b', 'c']))
+        )
+        self.assertEqual(expr_stream.bbox, None)
+        self.assertEqual(len(events), 3)
 
     def test_selection_expr_stream_dynamic_map(self):
         for element_type in [Scatter, Points]:
@@ -1066,13 +1343,13 @@ class TestExprSelectionStream(ComparisonTestCase):
             expr_stream = SelectionExpr(dmap)
 
             # Check stream properties
-            self.assertEqual(len(expr_stream._source_streams), 1)
-            self.assertIsInstance(expr_stream._source_streams[0], BoundsXY)
+            self.assertEqual(len(expr_stream.input_streams), 3)
+            self.assertIsInstance(expr_stream.input_streams[0], SelectionXY)
             self.assertIsNone(expr_stream.bbox)
             self.assertIsNone(expr_stream.selection_expr)
 
             # Simulate interactive update by triggering source stream
-            expr_stream._source_streams[0].event(bounds=(1, 1, 3, 4))
+            expr_stream.input_streams[0].event(bounds=(1, 1, 3, 4))
 
             # Check SelectionExpr values
             self.assertEqual(
